#include "i2c.h"

/* This function issues a start condition and
 * transmits the slave address + R/W bit
 *
 * Parameters:
 *      I2Cx --> the I2C peripheral e.g. I2C1
 *      address --> the 7 bit slave address
 *      direction --> the transmission direction can be:
 *                      I2C_Direction_Tranmitter for Master transmitter mode
 *                      I2C_Direction_Receiver for Master receiver
 */
void I2C_start(I2C_TypeDef* I2Cx, uint8_t address, uint8_t direction){
    // wait until I2C1 is not busy any more
    while(I2C_GetFlagStatus(I2Cx, I2C_FLAG_BUSY));

    // Send I2C1 START condition
    I2C_GenerateSTART(I2Cx, ENABLE);

    // wait for I2C1 EV5 --> Slave has acknowledged start condition
    while(!I2C_CheckEvent(I2Cx, I2C_EVENT_MASTER_MODE_SELECT));

    // Send slave Address for write
    I2C_Send7bitAddress(I2Cx, address, direction);

    /* wait for I2Cx EV6, check if
     * either Slave has acknowledged Master transmitter or
     * Master receiver mode, depending on the transmission
     * direction
     */
    if(direction == I2C_Direction_Transmitter){
        while(!I2C_CheckEvent(I2Cx, I2C_EVENT_MASTER_TRANSMITTER_MODE_SELECTED));
    }
    else if(direction == I2C_Direction_Receiver){
        while(!I2C_CheckEvent(I2Cx, I2C_EVENT_MASTER_RECEIVER_MODE_SELECTED));
    }
}

/* This function transmits one byte to the slave device
 * Parameters:
 *      I2Cx --> the I2C peripheral e.g. I2C1
 *      data --> the data byte to be transmitted
 */
void I2C_write(I2C_TypeDef* I2Cx, uint8_t data)
{
    // wait for I2C1 EV8 --> last byte is still being transmitted (last byte in SR, buffer empty), next byte can already be written
    while(!I2C_CheckEvent(I2Cx, I2C_EVENT_MASTER_BYTE_TRANSMITTING));
    I2C_SendData(I2Cx, data);
}

/* This function reads one byte from the slave device
 * and acknowledges the byte (requests another byte)
 */
uint8_t I2C_read_ack(I2C_TypeDef* I2Cx){
    // enable acknowledge of received data
    I2C_AcknowledgeConfig(I2Cx, ENABLE);
    // wait until one byte has been received
    while( !I2C_CheckEvent(I2Cx, I2C_EVENT_MASTER_BYTE_RECEIVED) );
    // read data from I2C data register and return data byte
    uint8_t data = I2C_ReceiveData(I2Cx);
    return data;
}

/* This function reads one byte from the slave device
 * and doesn't acknowledge the received data
 * after that a STOP condition is transmitted
 */
uint8_t I2C_read_nack(I2C_TypeDef* I2Cx){
    // disable acknowledge of received data
    // nack also generates stop condition after last byte received
    // see reference manual for more info
    I2C_AcknowledgeConfig(I2Cx, DISABLE);
    I2C_GenerateSTOP(I2Cx, ENABLE);
    // wait until one byte has been received
    while( !I2C_CheckEvent(I2Cx, I2C_EVENT_MASTER_BYTE_RECEIVED) );
    // read data from I2C data register and return data byte
    uint8_t data = I2C_ReceiveData(I2Cx);
    return data;
}

/* This function issues a stop condition and therefore
 * releases the bus
 */
void I2C_stop(I2C_TypeDef* I2Cx){

    // Send I2C1 STOP Condition after last byte has been transmitted
    I2C_GenerateSTOP(I2Cx, ENABLE);
    // wait for I2C1 EV8_2 --> byte has been transmitted
    while(!I2C_CheckEvent(I2Cx, I2C_EVENT_MASTER_BYTE_TRANSMITTED));
}



//
//I2C_TypeDef * I2C_Module;
//volatile uint8_t deviceAddress;
//volatile uint8_t dataByte1;
//volatile uint8_t dataByte0;
//volatile uint8_t receivedDataByte1;
//volatile uint8_t receivedDataByte0;
//volatile uint8_t i2cDirectionWrite;
//volatile uint8_t i2cByteCounter;
//volatile uint8_t i2cBusyFlag;
//
//void i2c_writeByte(uint8_t address, uint8_t byte) {
//    while (i2cBusyFlag) {
//    }
//    while (I2C_GetFlagStatus(I2C_Module, I2C_FLAG_BUSY)) {
//    }
//    deviceAddress = address;
//    dataByte0 = byte;
//    i2cDirectionWrite = 1;
//    i2cBusyFlag = 1;
//    i2cByteCounter = 1;
//    I2C_GenerateSTART(I2C_Module, ENABLE);
//}
//
//void i2c_writeTwoBytes(uint8_t address, uint8_t byte1, uint8_t byte0) {
//    while (i2cBusyFlag) {
//    }
//    while (I2C_GetFlagStatus(I2C_Module, I2C_FLAG_BUSY)) {
//    }
//    deviceAddress = address;
//    dataByte1 = byte1;
//    dataByte0 = byte0;
//    i2cDirectionWrite = 1;
//    i2cBusyFlag = 1;
//    i2cByteCounter = 2;
//    I2C_GenerateSTART(I2C_Module, ENABLE);
//}
//
//void i2c_readTwoBytes(uint8_t address) {
//    while (i2cBusyFlag) {
//    }
//    while (I2C_GetFlagStatus(I2C_Module, I2C_FLAG_BUSY)) {
//    }
//    deviceAddress = address;
//    i2cDirectionWrite = 0;
//    i2cBusyFlag = 1;
//    i2cByteCounter = 2;
//    I2C_AcknowledgeConfig(I2C_Module, ENABLE);
//    I2C_GenerateSTART(I2C_Module, ENABLE);
//}
//
//void i2c_create(I2C_TypeDef * I2Cx) {
//    I2C_Module = I2Cx;
//    i2cBusyFlag = 0;
//}
//
//uint16_t i2c_getData(void) {
//    return (receivedDataByte1 << 8) | receivedDataByte0;
//}
//
//// ISR
//void i2c_handleEventInterrupt(void) {
//    if (I2C_GetFlagStatus(I2C_Module, I2C_FLAG_SB) == SET) {
//        if (i2cDirectionWrite) {
//            // STM32 Transmitter
//            I2C_Send7bitAddress(I2C_Module, deviceAddress,
//                    I2C_Direction_Transmitter);
//        } else {
//            // STM32 Receiver
//            I2C_Send7bitAddress(I2C_Module, deviceAddress,
//                    I2C_Direction_Receiver);
//        }
//    } else if (I2C_GetFlagStatus(I2C_Module, I2C_FLAG_ADDR) == SET
//            || I2C_GetFlagStatus(I2C_Module, I2C_FLAG_BTF) == SET) {
//        I2C_ReadRegister(I2C_Module, I2C_Register_SR1);
//        I2C_ReadRegister(I2C_Module, I2C_Register_SR2);
//        if (i2cDirectionWrite) {
//            // STM32 Transmitter
//            if (i2cByteCounter == 2) {
//                I2C_SendData(I2C_Module, dataByte1);
//                i2cByteCounter--;
//            } else if (i2cByteCounter == 1) {
//                I2C_SendData(I2C_Module, dataByte0);
//                i2cByteCounter--;
//            } else {
//                I2C_GenerateSTOP(I2C_Module, ENABLE);
//                i2cBusyFlag = 0;
//            }
//        }
//    } else if (I2C_GetFlagStatus(I2C_Module, I2C_FLAG_RXNE) == SET) {
//        // STM32 Receiver
//        I2C_ReadRegister(I2C_Module, I2C_Register_SR1);
//        I2C_ReadRegister(I2C_Module, I2C_Register_SR2);
//        i2cByteCounter--;
//        if (i2cByteCounter == 1) {
//            I2C_AcknowledgeConfig(I2C_Module, DISABLE);
//            I2C_GenerateSTOP(I2C_Module, ENABLE);
//            receivedDataByte1 = I2C_ReceiveData(I2C_Module);
//        } else {
//            receivedDataByte0 = I2C_ReceiveData(I2C_Module);
//            i2cBusyFlag = 0;
//        }
//    }
//}
//
//// ISR
//void i2c_handleErrorInterrupt(void) {
//    I2C_GenerateSTOP(I2C_Module, ENABLE);
//    i2cBusyFlag = 0;
//
//    I2C_ClearFlag(I2C_Module, I2C_FLAG_AF);
//    I2C_ClearFlag(I2C_Module, I2C_FLAG_ARLO);
//    I2C_ClearFlag(I2C_Module, I2C_FLAG_BERR);
//}
